/*
 * lcdswitch.c : display string on LCD/read switch
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>

/* Some constants used later on */
#define TIMEOUT         2000
#define ENDPOINT        1
#define CONFIG          1

#define VID		0x12bf
#define	PID		0xff04

usb_dev_handle *findAndOpenUsb() {
  struct usb_bus *bus;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus; bus=bus->next) {
    struct usb_device *device;

    for (device = bus->devices; device; device = device->next) {
      struct usb_dev_handle *dev;

      if (device->descriptor.idVendor == VID && device->descriptor.idProduct == PID) {
	dev = usb_open(device);
	if (usb_set_configuration(dev,CONFIG) < 0) {
	  printf("Error setting configuration\n");
	  exit(1);
	}
	if (usb_claim_interface(dev, 0)) {
	  printf("Can't claim interface for communication.\n");
	  exit(1);
	}
	return dev;
      }
    }
  }
  return (usb_dev_handle *)NULL;
}

int readSwitch(usb_dev_handle *dev,int swtch) {
  unsigned char buffer[2];
  int len;

  // write read switch command to device
  buffer[0] = 0x02;
  buffer[1] = swtch;
  if (usb_bulk_write(dev,ENDPOINT,buffer,2,TIMEOUT) < 0) {
    printf("Error writing.\n");
    usb_close(dev);
    exit(2);
  }

  // read response
  len = usb_bulk_read(dev,ENDPOINT,buffer,1,TIMEOUT);
  if ( len != 1 ) {
    printf("Error reading\n");
    usb_close(dev);
    exit(2);
  }
  if (buffer[0] == 1) {
    printf("Switch closed\n");
  } else {
    printf("Switch open\n");
  }
  return (buffer[0]);
}

void sendString(usb_dev_handle *dev,char *string) {
  unsigned char buffer[20];
  int len, i;

  len = strlen(string);
  if (len > 16) {
    len = 16;
  }

  // write display message command to device
  buffer[0] = 0x01;
  buffer[1] = len+1;
  for (i = 0; i < len; i++) {
    buffer[2+i] = string[i];
  }
  if (usb_bulk_write(dev,ENDPOINT,buffer,2+len,TIMEOUT) < 0) {
    printf("Error writing.\n");
    usb_close(dev);
    exit(2);
  }

  // read response
  len = usb_bulk_read(dev,ENDPOINT,buffer,1,TIMEOUT);
  if ( len != 1 ) {
    printf("Error reading\n");
    usb_close(dev);
    exit(2);
  }
  if (buffer[0] == 1) {
    printf("Displayed successfull\n");
  }
}

void usage() {
  printf("Usage: lcdswitch --switch <nr>\n");
  printf("                 --display \"<string>\"\n");
  exit(255);
}

int main(int argc, char *argv[]) {
  usb_dev_handle *device;
  int display = 0;
  int swtch = 0;
  int rdswtch = 0;
  char *msg = "";
  int i;
  int result = 0;

  device = findAndOpenUsb();
  if (device == NULL) {
    printf("Device not found.\n");
    exit(1);
  }

  if (argc < 2) {
    usage();
  }

  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i],"--switch")) {
      if (++i < argc) {
        rdswtch++;
      	swtch=atoi(argv[i]);
	continue;
      } else {
        usage();
      }
    }
    if (!strcmp(argv[i],"--display")) {
      if (++i < argc) {
	display++;
      	msg=argv[i];
	continue;
      } else {
        usage();
      }
    }
    usage();
  }

  if (rdswtch) {
    result=readSwitch(device,swtch);
  }

  if (display) {
    sendString(device,msg);
  }

  exit(result);
}
