/*
 * version.c : find device and query for version information
 */

#include <stdio.h>
#include <stdlib.h>
#include <usb.h>

/* Some constants used later on */
#define TIMEOUT         2000
#define ENDPOINT        1
#define CONFIG          1

#define VID		0x12bf
#define	PID		0xff04

usb_dev_handle *findAndOpenUsb() {
  struct usb_bus *bus;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_get_busses(); bus; bus=bus->next) {
    struct usb_device *device;

    for (device = bus->devices; device; device = device->next) {
      struct usb_dev_handle *dev;

      if (device->descriptor.idVendor == VID && device->descriptor.idProduct == PID) {
	dev = usb_open(device);
	if (usb_set_configuration(dev,CONFIG) < 0) {
	  printf("Error setting configuration\n");
	  exit(1);
	}
	if (usb_claim_interface(dev, 0)) {
	  printf("Can't claim interface for communication.\n");
	  exit(1);
	}
	return dev;
      }
    }
  }
  return (usb_dev_handle *)NULL;
}

void getVersion(usb_dev_handle *dev) {
  unsigned char buffer[2];
  int len;

  // write version command to device
  buffer[0] = 0x00;
  if (usb_bulk_write(dev,ENDPOINT,buffer,1,TIMEOUT) < 0) {
    printf("Error writing.\n");
    usb_close(dev);
    exit(2);
  }

  // read response
  len = usb_bulk_read(dev,ENDPOINT,buffer,2,TIMEOUT);
  if ( len != 2 ) {
    printf("Error reading\n");
    usb_close(dev);
    exit(2);
  }
  printf("Firmware version: %d.%02d\n",buffer[1],buffer[0]);
}

int main(int argc, char *argv[]) {
  usb_dev_handle *device;

  device = findAndOpenUsb();
  if (device == NULL) {
    printf("Device not found.\n");
    exit(1);
  }
  getVersion(device);
}
